/**
 * Make directories.
 * This is an apeman task contribution plugin.
 * @module apeman-task-contrib-mkdir
 * @see https://github.com/apeman-repo/apeman
 */

"use strict";

var async = require('async'),
    mkdirp = require('mkdirp'),
    fs = require('fs'),
    argx = require('argx');

/**
 * Task worker function to make directories.
 * @function mkdir
 * @param {object} context - Task context.
 * @param {string} dirnames - Task argument values.
 * @param {object} [options] - Task options.
 * @param {function} callback - Callback when done.
 */
function mkdir(context, dirnames, options, callback) {

    var args = argx(arguments);
    context = args.shift('object');
    callback = args.pop('function');
    options = args.pop('object');
    dirnames = args.remain();
    var logger = context.logger;

    _rejectExisting(dirnames, function (dirnames) {
        async.eachSeries(dirnames, function (dirname, callback) {
            mkdirp(dirname, function (err) {
                if (!err) {
                    logger.debug('Directory generated: ' + dirname);
                }
                callback(err);
            });
        }, callback);
    });
}

function _rejectExisting(dirnames, callback) {
    async.reject(dirnames, fs.exists, callback);
}


module.exports = mkdir;
